
/*********************************************************************************************************
 Title  :   C  include file for the ................................. (_________.c)
 Author:    ........................ 
 E-mail:    ........................
 Homepage:  ........................
 Date:      20/Aug/2002
 Compiler:  AVR-GCC with AVR-AS
 MCU type:  any AVR MCU device
 Comments:  This software is FREE.
*********************************************************************************************************/

/*
   COMMENTS
*/




/********************************************************************************************************/
/*                                   PREPROCESSOR DIRECTIVES                                            */
/********************************************************************************************************/

#include <avr/io.h>
#include <avr/signal.h>
#include "rc_channel.h"



/* Some error checking */
#define RC_HIGH_LEVEL_TIMEOUT_VALUE          ( (RC_HIGH_LEVEL_TIMEOUT*1000)/((1000000000/F_CPU)*RC_TIMER_PRESCALER) )
#if  RC_HIGH_LEVEL_TIMEOUT_VALUE <= 0
#error "Prescaler value set too high"
#elif   RC_HIGH_LEVEL_TIMEOUT_VALUE >= (0xFFFFFFFF - 1)
#error  "Prescaler value set too low"
#endif
#define RC_ISR_HIGH_LEVEL_TIMEOUT_VALUE      ((RC_HIGH_LEVEL_TIMEOUT_VALUE/0xFF)+1)

#define RC_LOW_LEVEL_TIMEOUT_VALUE           ( (RC_LOW_LEVEL_TIMEOUT*1000)/((1000000000/F_CPU)*RC_TIMER_PRESCALER) )
#if  RC_HIGH_LEVEL_TIMEOUT_VALUE <= 0
#error "Prescaler value set too high"
#elif   RC_LOW_LEVEL_TIMEOUT_VALUE >= (0xFFFFFFFF - 1)
#error  "Prescaler value set too low"
#endif
#define RC_ISR_LOW_LEVEL_TIMEOUT_VALUE       ((RC_LOW_LEVEL_TIMEOUT_VALUE/0xFF)+1)




/* Definition of the prescaler bit settings according to where they will be used. */

#if RC_TIMER_PRESCALER == 1
#define RC_PRESCALER_BITS               RC_PRESCALE_BY_1 
#elif RC_TIMER_PRESCALER == 8
#define RC_PRESCALER_BITS               RC_PRESCALE_BY_8 
#elif  RC_TIMER_PRESCALER == 64
#define RC_PRESCALER_BITS               RC_PRESCALE_BY_64 
#elif  RC_TIMER_PRESCALER == 256
#define RC_PRESCALER_BITS               RC_PRESCALE_BY_256
#elif  RC_TIMER_PRESCALER == 1024
#define RC_PRESCALER_BITS               RC_PRESCALE_BY_1024  
#endif

 





/********************************************************************************************************/
/*                                   TYPE DEFINITIONS                                                   */
/********************************************************************************************************/





/********************************************************************************************************/
/*                                   GLOBAL VARIABLES                                                   */
/********************************************************************************************************/

volatile unsigned long   timer0_count=0;
volatile unsigned long   isr_limit = 0;



/********************************************************************************************************/
/*                                   LOCAL FUNCTION PROTOTYPES                                          */
/********************************************************************************************************/






/********************************************************************************************************/
/*                                   LOCAL FUNCTION DEFINITIONS                                         */
/********************************************************************************************************/






/********************************************************************************************************/
/*                                   PUBLIC FUNCTIONS                                                   */
/********************************************************************************************************/

unsigned int rc_get_pw(volatile unsigned char* port, unsigned char pin)
{

unsigned char  global_interupt_on=0, rc_timer_interupt_memory=0;
unsigned char high_counter=0, low_counter=0;


                /* Make the signal pin an input pin. */ 
                *(port-1) = ( *(port-1) & (~(1<<pin)) );

                /* Turn off the pull up resistor. */ 
                *(port) = ( *(port) & (~(1<<pin)) );

#if RC_CHANNEL_AUTO_INIT == 1

                /* Set the timer x interupt enable bit. */
                if(bit_is_set(RC_TIMER_INT_MASK_REGISTER,RC_TIMER_INT_ENABLE_BIT))
                 {
                    rc_timer_interupt_memory = 1; 	
                 }
                else { sbi(RC_TIMER_INT_MASK_REGISTER,RC_TIMER_INT_ENABLE_BIT); }  	

                /* Enable global interupts. */
                if(bit_is_set(SREG, 7)) { global_interupt_on=1; } else { asm("sei");  }   


#endif

                /* Counting is stoped. */
                RC_TIMER_CONTROL_REGISTER = 0;
                /* Reset the timer. */
                RC_TIMER_DATA_REGISTER = 0;
                /* Reset the counters ISR counter. */
                timer0_count = 0;
                /* Load the appropriate counter limit. */
                isr_limit = RC_ISR_HIGH_LEVEL_TIMEOUT_VALUE;

                
/*--------------------------------------------------------------------------------------------------*/
                /* Wait until the Signal line is low */
/*--------------------------------------------------------------------------------------------------*/
                RC_TIMER_CONTROL_REGISTER =  RC_PRESCALER_BITS;

while(1)
    {
        if(bit_is_clear(*(port-2),pin)) { low_counter++; } else{ low_counter=0; }
        if(low_counter >= RC_SIGNAL_SAMPLES) { low_counter=0; break; }
        /* Since TCCRx is not 0, if it reads 0 then the ISR signalled an overflow. */
        if(RC_TIMER_CONTROL_REGISTER == 0) { goto END; }
    }	


                /* Make sure that the signal line is not high, wait until it goes low. */
 /*               while(bit_is_set(*(port-2),pin)) {  if(isr_limit_reached == 1) { goto END; }  } */	

                /* Counting is stoped. */
                RC_TIMER_CONTROL_REGISTER = 0;
                /* Reset the timer. */
                RC_TIMER_DATA_REGISTER = 0;
                timer0_count = 0;
                /* Load the appropriate counter limit. */
                isr_limit = RC_ISR_LOW_LEVEL_TIMEOUT_VALUE;

/*--------------------------------------------------------------------------------------------------*/
/*   Wait until the Signal line goes high again.                                                    */
/*--------------------------------------------------------------------------------------------------*/

               /* Wait until the signal line goes high again, check for timeout. */
               RC_TIMER_CONTROL_REGISTER =  RC_PRESCALER_BITS;
                               
while(1)
    {
        if(bit_is_set(*(port-2),pin)) { high_counter++; }  else{ high_counter=0; }
        if(high_counter >= RC_SIGNAL_SAMPLES) { high_counter=0; break; }
        /* Since TCCRx is not 0, if it reads 0 then the ISR signalled an overflow. */
        if(RC_TIMER_CONTROL_REGISTER == 0) { goto END; }
    }	


                /* Counting is stoped. */
                RC_TIMER_CONTROL_REGISTER = 0;
                /* Reset the timer. */
                RC_TIMER_DATA_REGISTER = 0;
                timer0_count = 0;
                /* Load the appropriate counter limit. */
                isr_limit = RC_ISR_HIGH_LEVEL_TIMEOUT_VALUE;
                /* Start counting. */
                RC_TIMER_CONTROL_REGISTER = RC_PRESCALER_BITS;

/*------------------------------------------------------------------------------------------------*/
/*  Wait until the signal line goes low again so we can finish the measurement.                   */
/*------------------------------------------------------------------------------------------------*/

while(1)
    {
        if(bit_is_clear(*(port-2),pin)) { low_counter++; } else{ low_counter=0; }
        if(low_counter >= RC_SIGNAL_SAMPLES) { low_counter=0; break; }
        /* Since TCCRx is not 0, if it reads 0 then the ISR signalled an overflow. */
        if(RC_TIMER_CONTROL_REGISTER == 0) { goto END; }
    }	

                                                                        
/*------------------------------------------------------------------------------------------------*/
/* Now it is time to convert the timer count in to microseconds.                                  */
/*------------------------------------------------------------------------------------------------*/

               /* stop the timer. */
               RC_TIMER_CONTROL_REGISTER = 0;
               
               /* If the overflow flag is up then return zero. */
               timer0_count = (timer0_count<<8);
               timer0_count += RC_TIMER_DATA_REGISTER;
               timer0_count = ( (timer0_count*100*RC_TIMER_PRESCALER)/(F_CPU/10000) );
               
               if((timer0_count <= RC_MAX_WIDTH ) && (timer0_count >= RC_MIN_WIDTH))
                { 
                    return((unsigned int)timer0_count);
                }
                

END:                                

#if RC_CHANNEL_AUTO_INIT == 1
/* Restore the timer overflow interupt to its previous state. */ 
if(rc_timer_interupt_memory == 0) { cbi(RC_TIMER_INT_MASK_REGISTER,RC_TIMER_INT_ENABLE_BIT); }

/* Restore the global interupt bit to its previous state */
if(global_interupt_on == 0) { asm("cli"); } 
#endif

                            

return(0);
         
}




/********************************************************************************************************/
/*                                   INTERRUPT SERVICE ROUTINES                                         */
/********************************************************************************************************/

RC_INTERUPT_SERVICE_ROUTINE()
{

    if(timer0_count >= isr_limit ) { RC_TIMER_CONTROL_REGISTER = 0; }
    timer0_count++;

}




/*######################################################################################################*/
/*                                         T H E   E N D                                                */
/*######################################################################################################*/

