
/*********************************************************************************************************
 Title  :   C  main file for the ................................. (_________.c)
 Author:    ........................ 
 E-mail:    ........................
 Homepage:  ........................
 Date:      20/Aug/2002
 Compiler:  AVR-GCC with AVR-AS
 MCU type:  any AVR MCU device
 Comments:  This software is FREE.
*********************************************************************************************************/

/*
   COMMENTS
*/




/********************************************************************************************************/
/*                                   PREPROCESSOR DIRECTIVES                                            */
/********************************************************************************************************/

#include <avr/io.h>
#include <avr/signal.h>
#include "delay.h"
#include "rc_channel.h"

#define RC_SWITCH_PORT                     PORTB
#define RC_SWITCH_DDR_REGISTER             DDRB
#define RC_SIGNAL_PORT                     PORTB
#define RC_SIGNAL_PIN                      4
#define RC_SWITCH_OUTPUT_PIN               0 
#define RC_SWITCH_DEFAULT_STATE            0
#define RC_SWITCH_THRESHOLD_WIDTH          1500L     /* In microseconds, L makes the constant a long. */

/* Here i introduce some hysterisis in order to avoid oscilation around the threshold pulse width */
#define   RC_SWITCH_DOWN_THRESHOLD_WIDTH   ((RC_SWITCH_THRESHOLD_WIDTH*95)/100)
#define   RC_SWITCH_UP_THRESHOLD_WIDTH     ((RC_SWITCH_THRESHOLD_WIDTH*105)/100)


/********************************************************************************************************/
/*                                   MACRO DEFINITIONS                                                  */
/********************************************************************************************************/


/********************************************************************************************************/
/*                                   MAIN FUNCTION                                                      */
/********************************************************************************************************/
void main(void)
{
volatile unsigned int pulse_width=0;

/* Initialization */

/* Make the switch output pin an output */
RC_SWITCH_DDR_REGISTER |= (1<<RC_SWITCH_OUTPUT_PIN); 

/* Set the default switch state */
RC_SWITCH_PORT |= (RC_SWITCH_DEFAULT_STATE<<RC_SWITCH_OUTPUT_PIN);

/* Now delay in order for the receiver to power on. */
DELAY_MS(1000);


/* Main program code */


while(1){
                
/*
   The rc_get_pulse_width() function automaticaly makes the used pin an input, enables global interupts,
   timer specific interupt and before exiting it restores global interupt bit and the timer interupt
   to exactly their previus state. The used signal pin remains an input so if you want to make it an output
   you have to do it manualy. Becarefull because an output pin connected to the receiver is not a good thing.
   When the AVR is powered on all pins are inputs so there is no problem here. 
   Note that PORTx is used as a parameter to the function and not PINx !
   Actualy  rc_get_pulse_width() is a macro for rc_get_pw() and the address of PORTx is passed as a parameter.
   Have a look at rc_channel.h for more details.
*/
           pulse_width = rc_get_pulse_width(RC_SIGNAL_PORT, RC_SIGNAL_PIN);
           /* if the return value is zero then something went wrong or there was too much noise. */
           if(pulse_width==0) { continue; }
           /* If the time was greater than 1,5 ms (Threshold width) then make PB4 high. */
           if( pulse_width <= RC_SWITCH_DOWN_THRESHOLD_WIDTH ) { RC_SWITCH_PORT = 0; }
           else if( pulse_width >= RC_SWITCH_UP_THRESHOLD_WIDTH ) { RC_SWITCH_PORT = (1<<RC_SWITCH_OUTPUT_PIN); }
                 

        }        

return;
}

/*######################################################################################################*/
/*                                         T H E   E N D                                                */
/*######################################################################################################*/

